/*
 * Copyright 2023 teogor (Teodor Grigor)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     https://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package dev.teogor.ceres.ui.designsystem

import androidx.compose.foundation.shape.CircleShape
import androidx.compose.material3.ExperimentalMaterial3Api
import androidx.compose.material3.FilterChip
import androidx.compose.material3.FilterChipDefaults
import androidx.compose.material3.Icon
import androidx.compose.material3.ProvideTextStyle
import androidx.compose.runtime.Composable
import androidx.compose.ui.Modifier
import androidx.compose.ui.graphics.Color
import androidx.compose.ui.unit.dp
import dev.teogor.ceres.ui.icons.CeresIcons
import dev.teogor.ceres.ui.theme.MaterialTheme

/**
 * Ceres filter chip with included leading checked icon as well as text content slot.
 *
 * @param selected Whether the chip is currently checked.
 * @param onSelectedChange Called when the user clicks the chip and toggles checked.
 * @param modifier Modifier to be applied to the chip.
 * @param enabled Controls the enabled state of the chip. When `false`, this chip will not be
 * clickable and will appear disabled to accessibility services.
 * @param label The text label content.
 */
@Composable
@OptIn(ExperimentalMaterial3Api::class)
fun CeresFilterChip(
  selected: Boolean,
  onSelectedChange: (Boolean) -> Unit,
  modifier: Modifier = Modifier,
  enabled: Boolean = true,
  label: @Composable () -> Unit,
) {
  FilterChip(
    selected = selected,
    onClick = { onSelectedChange(!selected) },
    label = {
      ProvideTextStyle(value = MaterialTheme.typography.labelSmall) {
        label()
      }
    },
    modifier = modifier,
    enabled = enabled,
    leadingIcon = if (selected) {
      {
        Icon(
          imageVector = CeresIcons.Check,
          contentDescription = null,
        )
      }
    } else {
      null
    },
    shape = CircleShape,
    border = FilterChipDefaults.filterChipBorder(
      borderColor = MaterialTheme.colorScheme.onBackground,
      selectedBorderColor = MaterialTheme.colorScheme.onBackground,
      disabledBorderColor = MaterialTheme.colorScheme.onBackground.copy(
        alpha = CeresChipDefaults.DisabledChipContentAlpha,
      ),
      disabledSelectedBorderColor = MaterialTheme.colorScheme.onBackground.copy(
        alpha = CeresChipDefaults.DisabledChipContentAlpha,
      ),
      selectedBorderWidth = CeresChipDefaults.ChipBorderWidth,
    ),
    colors = FilterChipDefaults.filterChipColors(
      labelColor = MaterialTheme.colorScheme.onBackground,
      iconColor = MaterialTheme.colorScheme.onBackground,
      disabledContainerColor = if (selected) {
        MaterialTheme.colorScheme.onBackground.copy(
          alpha = CeresChipDefaults.DisabledChipContainerAlpha,
        )
      } else {
        Color.Transparent
      },
      disabledLabelColor = MaterialTheme.colorScheme.onBackground.copy(
        alpha = CeresChipDefaults.DisabledChipContentAlpha,
      ),
      disabledLeadingIconColor = MaterialTheme.colorScheme.onBackground.copy(
        alpha = CeresChipDefaults.DisabledChipContentAlpha,
      ),
      selectedContainerColor = MaterialTheme.colorScheme.primaryContainer,
      selectedLabelColor = MaterialTheme.colorScheme.onBackground,
      selectedLeadingIconColor = MaterialTheme.colorScheme.onBackground,
    ),
  )
}

/**
 * Ceres chip default values.
 */
object CeresChipDefaults {
  // TODO: File bug
  // FilterChip default values aren't exposed via FilterChipDefaults
  const val DisabledChipContainerAlpha = 0.12f
  const val DisabledChipContentAlpha = 0.38f
  val ChipBorderWidth = 1.dp
}
