/*
 * Copyright 2023 teogor (Teodor Grigor)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     https://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package dev.teogor.ceres.ui.designsystem

import androidx.compose.material3.FilledIconToggleButton
import androidx.compose.material3.IconButton
import androidx.compose.material3.IconButtonDefaults
import androidx.compose.runtime.Composable
import androidx.compose.ui.Modifier
import androidx.compose.ui.graphics.Color
import dev.teogor.ceres.ui.theme.MaterialTheme

/**
 * Ceres toggle button with icon and checked icon content slots. Wraps Material 3
 * [IconButton].
 *
 * @param checked Whether the toggle button is currently checked.
 * @param onCheckedChange Called when the user clicks the toggle button and toggles checked.
 * @param modifier Modifier to be applied to the toggle button.
 * @param enabled Controls the enabled state of the toggle button. When `false`, this toggle button
 * will not be clickable and will appear disabled to accessibility services.
 * @param icon The icon content to show when unchecked.
 * @param checkedIcon The icon content to show when checked.
 */
@Composable
fun CeresIconToggleButton(
  checked: Boolean,
  onCheckedChange: (Boolean) -> Unit,
  modifier: Modifier = Modifier,
  enabled: Boolean = true,
  icon: @Composable () -> Unit,
  checkedIcon: @Composable () -> Unit = icon,
) {
  // TODO: File bug
  // Can't use regular IconToggleButton as it doesn't include a shape (appears square)
  FilledIconToggleButton(
    checked = checked,
    onCheckedChange = onCheckedChange,
    modifier = modifier,
    enabled = enabled,
    colors = IconButtonDefaults.iconToggleButtonColors(
      checkedContainerColor = MaterialTheme.colorScheme.primaryContainer,
      checkedContentColor = MaterialTheme.colorScheme.onPrimaryContainer,
      disabledContainerColor = if (checked) {
        MaterialTheme.colorScheme.onBackground.copy(
          alpha = CeresIconButtonDefaults.DisabledIconButtonContainerAlpha,
        )
      } else {
        Color.Transparent
      },
    ),
  ) {
    if (checked) checkedIcon() else icon()
  }
}

/**
 * Ceres icon button default values.
 */
object CeresIconButtonDefaults {
  // TODO: File bug
  // IconToggleButton disabled container alpha not exposed by IconButtonDefaults
  const val DisabledIconButtonContainerAlpha = 0.12f
}
